from . import HtmlMetaHub

try:
    from pylons import c
except ImportError:
    from pylons import tmpl_context as c


def htmlmeta_setup(**kwargs):
    """Attaches a HtmlMetaHub to a request as 'c._htmlmeta'

        You'd probably have something like this in your base controller:

        from htmlmeta_hub.pylons_helpers import *
        class Handler(object):
            def __init__(self,request):
                h.htmlmeta_init(title='MySite', keywords='abc', description='awesome website')

        The helpers should allow you to easily and cleanly manage metadata within views/handlers and templates.
    """
    c._htmlmeta = HtmlMetaHub(**kwargs)
    return c._htmlmeta


def htmlmeta_set_http_equiv(k, v):
    """proxies HtmlMetaHub.set_http_equiv through c._htmlmeta"""
    c._htmlmeta.set_http_equiv(k, v)


def htmlmeta_set_link(k, v):
    """proxies HtmlMetaHub.set_link through c._htmlmeta"""
    c._htmlmeta.set_link(k, v)


def htmlmeta_set_name(k, v):
    """proxies HtmlMetaHub.set_name through c._htmlmeta"""
    c._htmlmeta.set_name(k, v)


def htmlmeta_set_other(k, v):
    """proxies HtmlMetaHub.set_other through c._htmlmeta"""
    c._htmlmeta.set_other(k, v)


def htmlmeta_set(k, v):
    """proxies HtmlMetaHub.set through c._htmlmeta"""
    c._htmlmeta.set(k, v)


def htmlmeta_get(k):
    """proxies HtmlMetaHub.get through c._htmlmeta"""
    return c._htmlmeta.get(k)


def htmlmeta_unset(k):
    """proxies HtmlMetaHub.unset through c._htmlmeta"""
    c._htmlmeta.unset(k)


def htmlmeta_as_html():
    """proxies HtmlMetaHub.as_html through c._htmlmeta"""
    return c._htmlmeta.as_html()



from metadata_utils import html_attribute_escape

_http_equivs = ('content-type', 'expires', 'refresh', )
_link_rels = ('canonical', 'link', )


class HtmlMetaHub(object):
    data_struct = None

    def __init__(self, **kwargs):
        """Sets up self.data_struct dict which we use for storage of metadata.

        This package is rather silly, but it affords a consistent way to manage metadata across multiple projects.
        """
        self.data_struct = {'http-equiv': {}, 'name': {}, 'other': {}, 'link': {}, }
        for key, value in kwargs.iteritems():
            self.set(key, value)

    def set_http_equiv(self, key, value):
        self.data_struct['http-equiv'][key] = value

    def set_link(self, rel, value):
        self.data_struct['link'][rel] = value

    def set_name(self, key, value):
        self.data_struct['name'][key] = value

    def set_other(self, key, value):
        self.data_struct['other'][key] = value

    def set(self, key, value):
        """ set uses (k,v) as there exists valid meta names which are not kwarg safe
        """
        if key.lower() in _http_equivs:
            self.data_struct['http-equiv'][key] = value
        elif key.lower() in _link_rels:
            self.data_struct['link'][key] = value
        else:
            self.data_struct['name'][key] = value

    def get(self, key):
        if key.lower() in _http_equivs:
            return self.data_struct['http-equiv'][key]
        elif key.lower() in _link_rels:
            return self.data_struct['link'][key]
        else:
            return self.data_struct['name'][key]

    def unset(self, key):
        if key.lower() in _http_equivs:
            if key in self.data_struct['http-equiv']:
                del self.data_struct['http-equiv'][key]
        elif key.lower() in _link_rels:
            if key in self.data_struct['link']:
                del self.data_struct['link'][key]
        else:
            if key in self.data_struct['name']:
                del self.data_struct['name'][key]

    def as_html(self):
        """helper function. prints out metadata for you.

        You'd probably call it like this in a Mako template:
            <head>
                ${meta.as_html()|n}
            </head>

        Notice that you have to escape under Mako.   For more information on mako escape options - http://www.makotemplates.org/docs/filtering.html
        """
        output = []
        for (k, v) in self.data_struct['http-equiv'].iteritems():
            output.append(u"""<meta http-equiv="%s" content="%s"/>""" % (html_attribute_escape(k), html_attribute_escape(v)))
        for (k, v) in self.data_struct['name'].iteritems():
            output.append(u"""<meta name="%s" content="%s"/>""" % (html_attribute_escape(k), html_attribute_escape(v)))
        for (k, v) in self.data_struct['other'].iteritems():
            output.append(u"""<meta %s="%s"/>""" % (html_attribute_escape(k), html_attribute_escape(v)))
        for (rel, v) in self.data_struct['link'].iteritems():
            output.append(u"""<link rel="%s" href="%s"/>""" % (html_attribute_escape(rel), html_attribute_escape(v)))
        return u"\n".join(output)



from . import HtmlMetaHub
from pyramid.threadlocal import get_current_request


def htmlmeta_setup(request=None, **kwargs):
    """Attaches a HtmlMetaHub to a request as 'request._htmlmeta'

        You'd probably have something like this in your base controller:

        from htmlmeta_hub.pyramid_helpers import *
        class Handler(object):
            def __init__(self,request):
                self.request = request
                h.htmlmeta_init(self.request, title='MySite', keywords='abc', description='awesome website')

        All the commands in the module accept an optional 'request' kwarg, but you should really pass it in.

        If no 'request' is submitted, it will call pyramid.threadlocal.get_current_request()

        The helpers should allow you to easily and cleanly manage metadata within views/handlers and templates.
    """
    request = request or get_current_request()
    request._htmlmeta = HtmlMetaHub(**kwargs)
    return request._htmlmeta


def htmlmeta_set_http_equiv(k, v, request=None):
    """proxies HtmlMetaHub.set_http_equiv through request._htmlmeta"""
    request = request or get_current_request()
    request._htmlmeta.set_http_equiv(k, v)


def htmlmeta_set_link(k, v, request=None):
    """proxies HtmlMetaHub.set_link through request._htmlmeta"""
    request = request or get_current_request()
    request._htmlmeta.set_link(k, v)


def htmlmeta_set_name(k, v, request=None):
    """proxies HtmlMetaHub.set_name through request._htmlmeta"""
    request = request or get_current_request()
    request._htmlmeta.set_name(k, v)


def htmlmeta_set_other(k, v, request=None):
    """proxies HtmlMetaHub.set_other through request._htmlmeta"""
    request = request or get_current_request()
    request._htmlmeta.set_other(k, v)


def htmlmeta_set(k, v, request=None):
    """proxies HtmlMetaHub.set through request._htmlmeta"""
    request = request or get_current_request()
    request._htmlmeta.set(k, v)


def htmlmeta_get(k, request=None):
    """proxies HtmlMetaHub.get through request._htmlmeta"""
    request = request or get_current_request()
    return request._htmlmeta.get(k)


def htmlmeta_unset(k, request=None):
    """proxies HtmlMetaHub.unset through request._htmlmeta"""
    request = request or get_current_request()
    return request._htmlmeta.unset(k)


def htmlmeta_as_html(request=None):
    """proxies HtmlMetaHub.as_html through request._htmlmeta"""
    request = request or get_current_request()
    return request._htmlmeta.as_html()
